import {Component, Input, OnInit, ViewChild} from '@angular/core';
import {WakeupTimeService} from "../wakeup-time.service";
import {WakeupTime} from "../wakeup-time";
import {MatMenuTrigger} from "@angular/material/menu";
import {MatDialog} from "@angular/material/dialog";
import {ConfirmationAlertDialogComponent} from "../confirmation-alert-dialog/confirmation-alert-dialog.component";
import {ThemePalette} from "@angular/material/core";
import {Util} from "../util";
import {ErrorHandlerService} from "../error-handler.service";

@Component({
  selector: 'app-calendar-table-entry',
  templateUrl: './calendar-table-entry.component.html',
  styleUrls: ['./calendar-table-entry.component.css']
})
export class CalendarTableEntryComponent implements OnInit {
  @Input() day = 0
  @Input() month = 0
  @Input() year = 0
  wakeUpTime: WakeupTime | undefined = undefined
  color: ThemePalette = undefined
  menuTopLeftPosition = {x: '0', y: '0'}
  // @ts-ignore
  @ViewChild(MatMenuTrigger, {static: true}) matMenuTrigger: MatMenuTrigger;
  inactive = true

  constructor(private wakeUpTimeService: WakeupTimeService, private dialog: MatDialog,
              private errorHandler: ErrorHandlerService) {
  }

  ngOnInit(): void {
    const identifier = this.formatDate(this.year, this.month, this.day)
    const observer = {
      next: (value: any) => this.updateValues(value),
      error: (err: any) => this.errorHandler.handleRetrievalError(err, this.dialog, identifier)
    }
    this.wakeUpTimeService.getWakeupTime(identifier)
      .subscribe(observer)
  }

  updateValues(value: WakeupTime | undefined): void {
    this.inactive = false
    this.wakeUpTime = value
    if (value === undefined) {
      this.color = undefined
    } else if (value.hour === undefined && value.minute === undefined) {
      this.color = "warn"
    } else {
      this.color = "primary"
    }
  }

  setWakeupTime(newValue: string): void {
    const newWakeupTime = Util.parseWakeupTime(newValue)
    const identifier = this.formatDate(this.year, this.month, this.day)
    const observer = {
      next: (_: any) => this.updateValues(newWakeupTime),
      error: (err: any) => this.errorHandler.handleWritingError(err, this.dialog, identifier)
    }

    this.wakeUpTimeService.setWakeupTime(identifier, newWakeupTime)
      .subscribe(observer)
  }

  getWakeupTimeStr(): string {
    if (this.wakeUpTime !== undefined && this.wakeUpTime.hour != undefined && this.wakeUpTime.minute !== undefined) {
      return `${this.wakeUpTime?.hour}:${this.wakeUpTime?.minute}`
    }
    return "00:00"
  }

  onRightClick(event: MouseEvent) {
    // preventDefault avoids to show the visualization of the right-click menu of the browser
    event.preventDefault();

    // we record the mouse position in our object
    this.menuTopLeftPosition.x = event.clientX + 'px';
    this.menuTopLeftPosition.y = event.clientY + 'px';

    // we open the menu
    this.matMenuTrigger.openMenu();

  }

  deleteWakeupTime() {
    const dialogRef = this.dialog.open(ConfirmationAlertDialogComponent, {
      data: {
        message: `Do you really want to delete the wakeup time
    for ${this.year}-${this.month}-${this.day}?`, title: "Deletion confirmation"
      }
    })
    const observer = {
      next: (_: any) => this.updateValues(undefined),
      error: (err: any) => this.errorHandler.handleDeletionError(err, this.dialog, this.formatDate(this.year, this.month, this.day))
    }


    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.wakeUpTimeService.deleteWakeupTime(this.formatDate(this.year, this.month, this.day))
          .subscribe(observer)
      }
    })
  }

  deactivateWakeupTime() {
    const dialogRef = this.dialog.open(ConfirmationAlertDialogComponent, {
      data: {
        message: `Do you really want to deactivate wakeup
    for ${this.year}-${this.month}-${this.day}?`, title: "Deactivation confirmation"
      }
    })
    const observer = {
      next: (_: any) => this.updateValues({"hour": undefined, "minute": undefined}),
      error: (err: any) => this.errorHandler.handleWritingError(err, this.dialog, this.formatDate(this.year, this.month, this.day))
    }

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.wakeUpTimeService.deactivateWakeupTime(this.formatDate(this.year, this.month, this.day))
          .subscribe(observer)
      }
    })

  }

  private formatDate(year: number, month: number, day: number): string {
    const monthPadded = Util.zeroPad(month + 1, 2)
    const dayPadded = Util.zeroPad(day, 2)
    return `${year}${monthPadded}${dayPadded}`
  }

}
