import {Component, Input, OnInit, ViewChild} from '@angular/core';
import {WakeupTime} from "../wakeup-time";
import {WakeupTimeService} from "../wakeup-time.service";
import {ThemePalette} from "@angular/material/core";
import {MatMenuTrigger} from "@angular/material/menu";
import {ConfirmationAlertDialogComponent} from "../confirmation-alert-dialog/confirmation-alert-dialog.component";
import {MatDialog} from "@angular/material/dialog";
import {Util} from "../util";
import {ErrorHandlerService} from "../error-handler.service";

@Component({
  selector: 'app-weekday-entry',
  templateUrl: './weekday-entry.component.html',
  styleUrls: ['./weekday-entry.component.css']
})
export class WeekdayEntryComponent implements OnInit {
  @Input() weekDay = ""
  @Input() shortName = ""
  wakeUpTime: WakeupTime | undefined = undefined
  color: ThemePalette = undefined
  wakeupStr = "00:00"
  menuTopLeftPosition = {x: '0', y: '0'}
  // @ts-ignore
  @ViewChild(MatMenuTrigger, {static: true}) matMenuTrigger: MatMenuTrigger
  inactive = true

  constructor(private wakeUpTimeService: WakeupTimeService, private dialog: MatDialog,
              private errorHandler: ErrorHandlerService) {
  }

  ngOnInit(): void {
    const observer = {
      next: (value: any) => this.updateValues(value),
      error: (err: any) => this.errorHandler.handleRetrievalError(err, this.dialog, this.weekDay)
    }
    this.wakeUpTimeService.getWakeupTime(this.shortName)
      .subscribe(observer)
  }

  updateValues(value: WakeupTime | undefined): void {
    this.inactive = false
    this.wakeUpTime = value
    if (value === undefined) {
      this.color = undefined
    } else {
      this.color = "primary"
    }
    this.wakeupStr = this.getWakeupTimeStr()
  }

  getWakeupTimeStr(): string {
    if (this.wakeUpTime !== undefined && this.wakeUpTime.hour != undefined && this.wakeUpTime.minute !== undefined) {
      const hour = Util.zeroPad(this.wakeUpTime.hour, 2)
      const minute = Util.zeroPad(this.wakeUpTime.minute, 2)
      return `${hour}:${minute}`
    }
    return "00:00"
  }

  setWakeupTime(newValue: string): void {
    const newWakeupTime: WakeupTime = Util.parseWakeupTime(newValue)
    const observer = {
      next: (_: any) => this.updateValues(newWakeupTime),
      error: (err: any) => this.errorHandler.handleWritingError(err, this.dialog, this.weekDay)
    }

    this.wakeUpTimeService.setWakeupTime(this.shortName, newWakeupTime)
      .subscribe(observer)
  }

  onRightClick(event: MouseEvent) {
    // preventDefault avoids to show the visualization of the right-click menu of the browser
    event.preventDefault();

    // we record the mouse position in our object
    this.menuTopLeftPosition.x = event.clientX + 'px';
    this.menuTopLeftPosition.y = event.clientY + 'px';

    // we open the menu
    this.matMenuTrigger.openMenu();

  }

  deleteWakeupTime() {
    const dialogRef = this.dialog.open(ConfirmationAlertDialogComponent, {
      data: {
        message: `Do you really want to delete the wakeup time
    for ${this.weekDay}?`, title: "Deletion confirmation"
      }
    })

    const observer = {
      next: (_: any) => this.updateValues(undefined),
      error: (err: any) => this.errorHandler.handleDeletionError(err, this.dialog, this.weekDay)
    }

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.wakeUpTimeService.deleteWakeupTime(this.shortName)
          .subscribe(observer)
      }
    })
  }

}
